% rbc2 computes Time Paths for the Stochastic
% Discrete Time Neoclassical Growth Modell, constant Technology
% Using Backward Integration. Reference: Brunner and Strulik (2001)
% Computes the Den Haan-Marcet test statistic
% calls the Function bestmu 2 & rbcfunc2.m

clear all; clc
global a d beta delta B ka ca ks cs g_exp vs cfinal kfinal tmax diskerror kstep cstep dir errror
%diskerror=10^(-8);

% Model Parameters (benchmark as in Cogley and Nason (1995)

lambda=0; %0.004;
a=0.344;
delta=0.021;
gamma=0.0037;
beta=1.03^-0.25;
d=-(1-a)/a; B=(gamma/(1-a))^d;

%Steady-State
g_exp=exp(lambda);
cs=((g_exp/beta-(1-delta))/(a*B))^(1/d)
ks=cs/(1-delta-g_exp+B.*cs^d);
Ls=(gamma.*cs./(1-a)).^(-1/a).*ks;

% Eigenvalues
x1=B/g_exp*cs^d+(1-delta);
x2=(B/g_exp*d*ks*cs^(d-1)-1)*cs/ks;
x3= beta/g_exp*((1-delta)/cs^2+a*B*(1-d)*cs^(d-2));
J21=0; J22=1/(cs^2*x3);
J=[x1 x2; J21 J22];
[eigvec,eigval]=eig(J);
[i,j]=min(abs(eig(J)));
vs=eigvec(:,j);                         % eigenvector for...
eigv=eig(J);eig1=eigv(j)                % ...stable eigenvalue (<1)


for sample=1:1:250                    %number of samples
%********** Initialization ******************************************
T=500;                         %length of one sample
T=T+1;
clear BB BE AA A xinst BB2 BE2 AA2 A2 xinst2 tt kk cc e g_t
tt=zeros(T,1); kk=zeros(T,1); cc=zeros(T,1);e=zeros(T,1);g_t=zeros(T,1);


stand_dev=0.03 %0.01; %097;
epsilon=randn(T,1).*stand_dev;
g_t=exp(epsilon);
tmax=200
j=1;

clc
%********* Main Loop *************************************************
while j<T+1
tt(j)=j;

if j==1
    kfinal=ks/g_t(1);
else
    kfinal=kk(j)/g_t(j);
end %if

%*********** Determine Direction *****************************************
if kfinal<ks
dir=1;                              % iteration in direction of smaller k's
mu0=-5.9*10^(-3);
else
dir=2;                              % larger k's
mu0=5.9*10^(-3);
end

%best mu's: for 0.01: 5.9*10^-3; for 0.02:

mu=fzero('bestmu2',mu0);

kk(j)=kfinal;cc(j)=cfinal;
ready_sample=sample
j=j+1
kk(j)=kfinal+kstep;;
merkmu(j)=mu;
end %while loop

kk(end)=[];
kk(1)=[];cc(1)=[];tt(1)=[];g_t(1)=[];

for i=2:1:length(cc)
e(i)=beta./g_t(i).*(a.*B.*cc(i).^(d-1)+(1-delta)./cc(i))-1./cc(i-1);
end
e(1)=[];

e=e(4:end);
rx=length(e);
xinst=[kk(3:end-1) kk(2:end-2) kk(1:end-3) g_t(3:end-1) g_t(2:end-2) g_t(1:end-3)];
xinst = [ones(rx,1) xinst];
for i = 1:rx,
   BB(i,:) = kron(e(i,:), xinst(i,:));
end;
BE = mean(BB);
AA =  BB' * BB;
A = AA / rx;
dmstat(sample) = rx * BE * inv(A) * BE';

save c:\docs\paper\stoch\sig003.mat kk cc tt mean_k ks cs T
break

%save c:\docs\paper\stoch\sig001.mat kk cc tt mean_k ks cs T

%save c:\docs\paper\stoch\sig001.mat dmstat dmstat2 dmstat3 mean_k

%********** Additional Regression on Autocorrelation of e *****************
for i=2:1:length(e)
    emin(i)=e(i-1);
end
[b,bint,r,rint,stats] =regress(e,emin')

%********* Additional Computations ****************************************
L=(gamma.*cc./(1-a)).^(-1/a).*kk;

zaehl=0
for i=1:1:250
    if dmstat(i)<2.16735
        zaehl=zaehl+1
    end
end
zaehl/250*100



kk=kk./ks;
cc=cc./cs;

%Investment
for i=2:1:length(kk)
I(i-1)=kk(i)-kk(i-1);
t(i-1)=tt(i-1);
end

(mean(mean_k)-ks)/ks*100
(mean(mean_c)-cs)/cs*100

zaehl=0
for i=1:1:137
    if dmstat(i)>14.0671
        zaehl=zaehl+1
    end
end
zaehl/137*100
% ********* distribution of realizations ********************************
aa=I;

aa=cc;
edges=linspace(min(aa),max(aa),25);
n=histc(aa,edges);
n=n/T*100;

set(gcf,'PaperUnits', 'centimeters','PaperType','a4letter','Units','centimeters','PaperPosition', [1 1 16 5],'Position',[1 1 16 5],'Color','w');
set(0,'DefaultTextColor','k')

subplot(1,2,1)
plot(edges,n,'b--')
hold on
set(gca,'FontName','Times','FontSize',10);
axis([-Inf +Inf -Inf +Inf])
set(gca,'Color','w','XColor','k','YColor','k','FontName','Times','FontSize',10);

subplot(1,2,2)
plot(kk,cc,'b.','MarkerSize',1)
hold on
set(gca,'FontName','Times','FontSize',10);
axis([-Inf +Inf -Inf +Inf])
set(gca,'Color','w','XColor','k','YColor','k','FontName','Times','FontSize',10);

break
gtext('{\it k/k^*}')	
gtext('{\it c/c^*}')	
gtext('{\it c/c^*}')
ylabel('{density of \it c/c^*}')

print -dpdf c:\docs\paper\stoch\rbc5.pdf