% rbc3.m
% Computes Time Paths for the Stochastic
% Discrete Time Neoclassical Growth Modell,
% Using Backward Iteration. Reference: Brunner and Strulik (2001)
% autocorrolated shocks, 2 dimensional manifold
% Calls the Function bestmu3.m

clear all, clc
global a d beta gamma delta B ka ca kappas Zfinal ks cs Zs g_exp vs Za rho muz


% Parameter

a  	= 0.344;
beta=1.03^-0.25;
delta  	= 0.021; 
theta = 1;
rho		= 0.96;
gamma=0.0037;

lambda=0.004;
d=-(1-a)/a; B=(gamma/(1-a))^d;

g_exp=exp(lambda);

Zs=1;
cs=((g_exp/beta-(1-delta))/(a*B))^(1/d)
ks=cs/(1-delta-g_exp+B.*cs^d);
Ls=(gamma.*cs./(1-a)).^(-1/a).*ks;
ys = Zs.*ks.^a.*Ls.^(1-a);
kappas=ks/Ls;

% Initial Value
x1=B/g_exp*cs^d+(1-delta);
x2=(B/g_exp*d*ks*cs^(d-1)-1)*cs/ks;
x3= beta/g_exp*((1-delta)/cs^2+a*B*(1-d)*cs^(d-2));
J21=0; J22=1/(cs^2*x3);
J=[x1 x2 0; J21 J22 0; 0 0 rho];
[eigvec,eigval]=eig(J);
vs1=eigvec(:,2); 						%stable eigenvectors
vs3=eigvec(:,3);

Zfinal=1.01;
muz=28.2*10^-7;
mu0=0.0005;
mu=fzero('bestmu3',mu0);


tmax=200;
t=zeros(1,tmax); c=zeros(1,tmax); k=zeros(1,tmax); Z=zeros(1,tmax);kappa=zeros(1,tmax);
Zfinal=1.01;

t(1)=0;
kappa(1)=kappas+mu;
k(1)=ks;
c(1)=cs;
Z(1)=Zs+muz;

% Backward Integration 
i=1; stop=1;

while stop==1 
   t(i+1)=t(i)+1;
   Z(i+1)=(Z(i))^(1/rho);
   c(i+1)=g_exp./beta.*c(i).*(a.*Z(i).*kappa(i)^(a-1)+(1-delta) )^-1;
   kappa(i+1)=(gamma*c(i+1)/(1-a)/Z(i+1))^(1/a);
   k(i+1)=(c(i+1)+g_exp.*k(i))./(Z(i+1).*kappa(i+1)^(a-1)+(1-delta));

   if Z(i+1)<Zfinal & i<tmax & isnan(k(i+1))==0 
      stop=1;
      i=i+1;
   else
      stop=0;
   end 
   
end %while loop


% Reversion of Solution Sequences
t(i+1:tmax)=[]; c(i+1:tmax)=[]; k(i+1:tmax)=[]; Z(i+1:tmax)=[];
t=max(t)-t;
t=t(length(t):-1:1); c=c(length(c):-1:1); k=k(length(k):-1:1); 
Z=Z(length(Z):-1:1); 
%I=I(length(I):-1:1);

% Impulse Response Functions
L=(gamma.*c./(1-a)./Z).^(-1/a).*k;
y = Z.*k.^a.*L.^(1-a);

%devy=(y-ys)*100/ys;
devZ=(Z-Zs)*100/Zs; devk=(k-ks)*100/ks; devc=(c-cs)*100/cs;
devy=(y-ys)*100/ys;
%devI=(I-d*ks)./(d*ks).*100;

set(gcf,'PaperUnits', 'centimeters','PaperType','a4letter','Units','centimeters','PaperPosition', [1 1 16 5],'Position',[1 1 16 5],'Color','w');
set(0,'DefaultTextColor','k')
plot3(devk,devZ,devc,'b-')
hold on
plot3(0,0,0,'b*')
plot3(devk(1),devZ(1),devc(1),'b.')
set(gca,'FontName','Times','FontSize',10);
axis([-Inf +Inf -Inf +Inf -Inf +Inf])
set(gca,'Color','w','XColor','k','YColor','k','ZColor','k','FontName','Times','FontSize',10);
grid on
view([37.5 30])

break

plot(t,devy,'b-')
axis([0 +Inf -Inf +Inf])
set(gca,'Color','w','XColor','k','YColor','k','FontName','Times','FontSize',10);
hold on

%pause

end %dir
break


subplot(1,2,1)
plot(t,devZ,'b-',t,devc,'b-',t,devk,'b-')
axis([0 50 -Inf +Inf])
set(gca,'Color','w','XColor','k','YColor','k','FontName','Times','FontSize',10);
axis([0 50 0 1.5])


break
